#ifndef R2C2_3D_LAYOUT_H_
#define R2C2_3D_LAYOUT_H_

#include <sigc++/trackable.h>
#include <msp/gl/instancescene.h>
#include <msp/gl/light.h>
#include <msp/gl/lighting.h>
#include <msp/gl/simplescene.h>
#include "libr2c2/layout.h"
#include "catalogue.h"
#include "typemap.h"

namespace R2C2 {

class Object3D;
class Utility3D;

class Layout3D: public sigc::trackable
{
public:
	typedef std::map<Object *, Object3D *> ObjectMap;

private:
	Layout &layout;
	Catalogue3D catalogue;
	ObjectMap objects;
	std::set<Utility3D *> utilities;
	Msp::GL::InstanceScene scene;
	Msp::GL::SimpleScene ep_scene;
	Msp::GL::InstanceScene path_scene;
	Msp::GL::Light sun;
	Msp::GL::Lighting lighting;

public:
	Layout3D(Layout &);
	~Layout3D();

	Layout &get_layout() const { return layout; }
	Catalogue3D &get_catalogue() { return catalogue; }
	const Catalogue3D &get_catalogue() const { return catalogue; }

	void add(Object3D &);
	Object3D &get_3d(Object &) const;
	const ObjectMap &get_all() const { return objects; }
	void remove(Object3D &);

	template<typename T>
	typename TypeMap3D<T>::Type3D &get_3d(T &o) const
	{ return dynamic_cast<typename TypeMap3D<T>::Type3D &>(get_3d(static_cast<Object &>(o))); }

	void add(Utility3D &);
	void remove(Utility3D &);

	Msp::GL::Scene &get_scene() { return scene; }
	Msp::GL::Scene &get_endpoint_scene() { return ep_scene; }
	Msp::GL::Scene &get_path_scene() { return path_scene; }
	const Msp::GL::Light &get_sun() const { return sun; }
	const Msp::GL::Lighting &get_lighting() const { return lighting; }

	void tick();

private:
	void object_added(Object &);
	void object_removed(Object &);
};

} // namespace R2C2

#endif

