#include <cmath>
#include <msp/gl/matrix.h>
#include <msp/gl/renderer.h>
#include "axle.h"
#include "bogie.h"
#include "layout.h"
#include "rod.h"
#include "vehicle.h"
#include "vehicletype.h"

using namespace std;
using namespace Msp;

namespace R2C2 {

Vehicle3D::Vehicle3D(Layout3D &l, Vehicle &v):
	Object3D(l, v),
	GL::ObjectInstance(*l.get_catalogue().get_3d(v.get_type()).get_body_object()),
	vehicle(v),
	type(layout.get_catalogue().get_3d(vehicle.get_type()))
{
	unsigned n_axles = vehicle.get_type().get_axles().size();
	for(unsigned i=0; i<n_axles; ++i)
		if(type.get_axle_object(i))
			children.push_back(new Axle3D(*this, i));

	unsigned n_bogies = vehicle.get_type().get_bogies().size();
	for(unsigned i=0; i<n_bogies; ++i)
		if(type.get_bogie_object(i))
			children.push_back(new Bogie3D(*this, i));

	unsigned n_rods = vehicle.get_type().get_rods().size();
	for(unsigned i=0; i<n_rods; ++i)
		if(type.get_rod_object(i))
			children.push_back(new Rod3D(*this, i));

	layout.get_scene().add(*this);
	for(vector<VehiclePart3D *>::const_iterator i=children.begin(); i!=children.end(); ++i)
		layout.get_scene().add(**i);
}

Vehicle3D::~Vehicle3D()
{
	layout.get_scene().remove(*this);
	for(vector<VehiclePart3D *>::iterator i=children.begin(); i!=children.end(); ++i)
	{
		layout.get_scene().remove(**i);
		delete *i;
	}
}

Vector Vehicle3D::get_node() const
{
	Vector p = vehicle.get_position();
	return Vector(p.x, p.y, p.z+0.01+vehicle.get_type().get_height());
}

bool Vehicle3D::is_visible() const
{
	return vehicle.is_placed();
}

void Vehicle3D::moved()
{
	Object3D::moved();

	for(vector<VehiclePart3D *>::const_iterator i=children.begin(); i!=children.end(); ++i)
		(*i)->update_matrix();
}

void Vehicle3D::render(GL::Renderer &renderer, const GL::Tag &tag) const
{
	if(!is_visible())
		return;

	ObjectInstance::render(renderer, tag);
}

void Vehicle3D::setup_render(GL::Renderer &renderer, const GL::Tag &) const
{
	renderer.matrix_stack() *= matrix;
	if(vehicle.get_type().get_rotate_object())
		renderer.matrix_stack() *= GL::Matrix::rotation(Angle::half_turn(), 0, 0, 1);
}

} // namespace R2C2
