#ifndef LIBR2C2_ROUTE_H_
#define LIBR2C2_ROUTE_H_

#include <map>
#include <set>
#include <string>
#include <sigc++/trackable.h>
#include <msp/datafile/objectloader.h>
#include "trackchain.h"

namespace R2C2 {

class Layout;
class Object;
class Track;
class TrackIter;
class Zone;


class Route: public TrackChain
{
public:
	typedef std::map<unsigned, int> TurnoutMap;

	class Loader: public Msp::DataFile::ObjectLoader<Route>
	{
	private:
		TurnoutMap turnouts;

	public:
		Loader(Route &);
	private:
		virtual void finish();
		void name(const std::string &);
		void turnout(unsigned, unsigned);
	};

private:
	bool temporary;
	TurnoutMap turnouts;

public:
	Route(Layout &);
	~Route();

	void set_temporary(bool);
	bool is_temporary() const { return temporary; }
	void update_turnouts();
private:
	void update_turnout(Track &);
public:
	int get_turnout(unsigned) const;
	unsigned get_path(Track &) const;
	const std::map<unsigned, int> &get_turnouts() const { return turnouts; }
private:
	void add_turnout(Track &, unsigned);
	virtual void on_track_added(Track &);

public:
	void save(std::list<Msp::DataFile::Statement> &) const;
	virtual Msp::DataFile::Statement save_reference() const;

	static Route *find(const TrackIter &, Track &);
	static Route *find(const TrackIter &, const Route &);
	static Route *find(const TrackIter &, const Zone &);
	static Route *find(const TrackIter &, const std::set<Track *> &);
};

} // namespace R2C2

#endif
